// Copyright (C) 2022 RozK
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _RK_ENGINE_RENDER_OPENGLES_H
#define _RK_ENGINE_RENDER_OPENGLES_H

#include "../render.hpp"
#include "render_context.hpp"
#include <GLES3/gl32.h>
#include <GLES3/gl3ext.h>
#include <GLES3/gl3platform.h>

enum : rk_uint {
    RK_VERTICES_BINDING = 0,
    RK_PARAMS_BINDING = 1
};

struct rk_shader {
    GLuint vertex;
    GLuint fragment;
    GLuint program;
};

struct rk_texture {
    rk_uint slot;
    rk_uint nlevels;
    GLint sampler;
    GLuint texture;
};

struct rk_triangles {
    rk_uint size;
    GLuint array;
    GLuint vertices;
};

struct rk_vertices {
    rk_uint vertex_size;
    rk_uint layout;
    GLuint array;
    GLuint vertices;
    GLuint indices;
};

typedef void (*rk_packer_fn)(rk_ubyte * const __restrict, rk_ubyte const * const __restrict);

struct rk_packer {
    rk_packer_fn pack;
    rk_uint dst_incr;
    rk_uint src_incr;
};

struct rk_command {
    GLuint nvertices;
    GLuint ninstances;
    GLuint base_index;
    GLint base_vertex;
    GLuint base_instance;
};

struct rk_batch {
    rk_uint size;
    rk_uint nparams;
    rk_uint params_size;
    rk_uint packed_size;
    rk_ushort * indices;
    rk_command * commands;
    rk_packer * packers;
    rk_ubyte * params;
    GLuint params_buffer;
    GLuint commands_buffer;
};

struct rk_vec3_short {
    rk_short x;
    rk_short y;
    rk_short z;
    rk_short pad;
};

#endif // _RK_ENGINE_RENDER_OPENGLES_H
